#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2019 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
mediawikiTool的基本命令框架
@module base_cmd
@file base_cmd.py
"""
import os
import sys
import threading
import traceback
import time
import subprocess
import warnings
from collections import Iterator
from HiveNetLib.generic import CResult
from HiveNetLib.simple_i18n import _, get_global_i18n  # 装载全局的i18n快捷函数
from HiveNetLib.base_tools.run_tool import RunTool
from HiveNetLib.simple_queue import MemoryQueue, Empty
from HiveNetLib.prompt_plus import PromptPlus
from HiveNetLib.base_tools.string_tool import StringTool
from HiveNetLib.simple_xml import SimpleXml
# 根据当前文件路径将包路径纳入，在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))


__MOUDLE__ = 'base_cmd'  # 模块名
__DESCRIPT__ = u'mediawikiTool的基本命令框架'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2019.12.04'  # 发布日期


class CmdBaseFW(object):
    """
    命令处理类框架类
    """

    #############################
    # 构造函数
    #############################
    def __init__(self, **kwargs):
        """
        构造函数

        @param {kwargs} - 传入初始化参数字典（config.xml的init_para字典）
            参数中可以设置的特殊参数如下：
            is_async=False {bool}  - 是否异步模式，如果设置为True，将会通过新启动线程的方式执行命令函数
                命令函数可以通过全局全局打印队列'CMD_BASE_PRINT_QUEUE'将打印信息输出给主进程打印
            is_i18n=False {bool} - 是否对返回输出的信息进行国际化转换（如果需要父类进行国际化，则应设置为True）
            ignore_case=False {bool} - 匹配命令是否忽略大小写(视需要传入，默认不区分大小写)

        @throws {Exception} - 如果初始化异常会抛出异常
        """
        self._kwargs = kwargs  # 初始化传入字典
        self._console_global_para = RunTool.get_global_var('CONSOLE_GLOBAL_PARA')
        self._init(**kwargs)

    def cmd_dealfun(self, message='', cmd='', cmd_para=''):
        """
        命令处理函数

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {iter} - 执行命令完成后要输到屏幕的内容，必须iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult的iter对象
            注2：如果直接return，信息将传递不到prompt的处理函数中，暂时未搞清楚原因
        """
        warnings.simplefilter('ignore', ResourceWarning)  # 指定屏蔽资源警告
        if 'is_async' in self._kwargs.keys() and self._kwargs['is_async']:
            # 异步模式，通过新启动线程方式执行函数
            self._queue = RunTool.get_global_var('CMD_BASE_PRINT_QUEUE')
            if self._queue is None:
                # 设置一个新的队列
                self._queue = MemoryQueue()

            # 启动命令执行线程
            self._thread_running = True
            _thread = threading.Thread(
                target=self._asyn_thread_fun,
                args=(message, cmd, cmd_para, self._kwargs),
                name='Thread-Cmd-DealFun'
            )
            _thread.setDaemon(True)
            _thread.start()

            # 遍历队列以及检查线程是否结束
            while self._thread_running or not self._queue.empty():
                # 从队列获取对象并返回外部打印
                try:
                    _item = self._queue.get(block=False)
                    if type(_item) == CResult:
                        yield _item.msg
                    else:
                        yield str(_item)
                except Empty:
                    time.sleep(0.01)

            # 处理完成，最终返回空
            yield ''
            return
        else:
            # 同步模式，直接执行函数处理
            _result = self._cmd_dealfun(message=message, cmd=cmd, cmd_para=cmd_para, **self._kwargs)
            if 'is_i18n' in self._kwargs.keys() and self._kwargs['is_async']:
                # 需要国际化处理
                if isinstance(_result, Iterator):
                    # 是迭代器
                    for _item in _result:
                        yield self._i18n_result_obj(_item)
                else:
                    yield self._i18n_result_obj(_result)
            else:
                # 不需要国际化处理，直接执行函数即可
                if isinstance(_result, Iterator):
                    for _item in _result:
                        yield _item
                else:
                    yield _result
            # 增加一个换行
            yield '\r\n'

    #############################
    # 内部处理函数
    #############################

    def _i18n_result_obj(self, result):
        """
        公共的处理返回结果国际化处理函数

        @param {string|CResult|iter} result - 可能是三种对象的其中一种
        """
        _type = type(result)
        if _type == CResult:
            # 错误对象
            result.msg = _(result.i18n_msg_id, result.i18n_msg_paras)
            return result
        elif _type == str:
            # 字符串
            return _(result)
        elif isinstance(result, Iterator):
            # 是迭代器
            for _result in result:
                # 回调自己进行处理
                yield self._i18n_result_obj(_result)
        else:
            # 字符串处理
            return _(result)

    def _asyn_thread_fun(self, message='', cmd='', cmd_para='', kwargs=dict()):
        try:
            _result = self._cmd_dealfun(message=message, cmd=cmd, cmd_para=cmd_para, **kwargs)
            _i18n_result = ''
            if 'is_i18n' in self._kwargs.keys() and self._kwargs['is_async']:
                _i18n_result = self._i18n_result_obj(_result)
            else:
                # 不需要国际化处理
                _i18n_result = _result

            # 判断返回值并区别处理
            if isinstance(_i18n_result, Iterator):
                # 是迭代器
                for _item in _i18n_result:
                    self._queue.put(_item)
            elif _i18n_result != '':
                self._queue.put(_i18n_result)
        except Exception as err:
            # 出现异常，将异常的打印信息放入显示队列
            _print_str = 'Call Excepiton: %s\r\n%s' % (
                str(err), traceback.format_exc()
            )
            self._queue.put(_print_str)
        finally:
            self._thread_running = False

    #############################
    # 需具体实现类覆盖实现的类
    #############################
    def _init(self, **kwargs):
        """
        实现类需要覆盖实现的初始化函数

        @param {kwargs} - 传入初始化参数字典（config.xml的init_para字典）

        @throws {exception-type} - 如果初始化异常应抛出异常
        """
        pass

    def _cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        实现类需要覆盖实现的命令处理函数

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）
        @param {kwargs} - 传入的主进程的初始化kwargs对象

        @return {string|iter_string} - 执行命令完成后要输到屏幕的内容，可以是iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult对象
        """
        pass


class CmdDefault(CmdBaseFW):
    """
    匹配不到命令时执行的处理函数类
    """
    #############################
    # 实际处理函数
    #############################

    def _cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        默认命令处理函数(未匹配到的命令)

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）
        @param {kwargs} - 传入的主进程的初始化kwargs对象

        @return {string|iter_string} - 执行命令完成后要输到屏幕的内容，可以是iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult对象
        """
        return _("'$1' is not support command!", cmd)


class CmdOnAbort(CmdBaseFW):
    """
    客户取消本次输入执行的函数类
    Ctrl + C : abort,取消本次输入
    """

    #############################
    # 实际处理函数
    #############################
    def _cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        Ctrl + C : abort,取消本次输入

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）
        @param {kwargs} - 传入的主进程的初始化kwargs对象

        @return {string|iter_string} - 执行命令完成后要输到屏幕的内容，可以是iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult对象
        """
        return ''


class CmdOnExit(CmdBaseFW):
    """
    客户关闭命令行执行的函数类
    Ctrl + D : exit,关闭命令行
    """

    #############################
    # 实际处理函数
    #############################
    def _cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        Ctrl + D : exit,关闭命令行

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）
        @param {kwargs} - 传入的主进程的初始化kwargs对象

        @return {string|iter_string} - 执行命令完成后要输到屏幕的内容，可以是iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult对象
        """
        _tip = _('You will shutdown $1 console, continue?(y/N)',
                 self._console_global_para['name'])
        _back = input(_tip)
        if _back.upper() == 'Y':
            # 退出
            return CResult(code='10101', msg='Exit $1 Console', i18n_msg_paras=(self._console_global_para['name'],))
        else:
            # 取消退出
            return CResult(code='00000', msg='Cancel Exit')


class CommonCmd(CmdBaseFW):
    """
    通用命令处理
    支持help、syscmd命令
    """
    #############################
    # 构造函数，在里面增加函数映射字典
    #############################

    def _init(self, **kwargs):
        """
        实现类需要覆盖实现的初始化函数

        @param {kwargs} - 传入初始化参数字典（config.xml的init_para字典）

        @throws {exception-type} - 如果初始化异常应抛出异常
        """
        self._CMD_DEALFUN_DICT = {
            'help': self._help_cmd_dealfun,
            'setlanguage': self._setlanguage_cmd_dealfun,
            'syscmd': self._syscmd_cmd_dealfun,
            'pwd': self._pwd_cmd_dealfun,
            'cd': self._cd_cmd_dealfun
        }
        self._console_global_para = RunTool.get_global_var('CONSOLE_GLOBAL_PARA')

    #############################
    # 实际处理函数
    #############################
    def _cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        通用处理函数，通过cmd区别调用实际的处理函数

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）
        @param {kwargs} - 传入的主进程的初始化kwargs对象

        @return {string|iter_string} - 执行命令完成后要输到屏幕的内容，可以是iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult对象
        """
        # 获取真实执行的函数
        _real_dealfun = None  # 真实调用的函数
        if 'ignore_case' in kwargs.keys() and kwargs['ignore_case']:
            # 区分大小写
            if cmd in self._CMD_DEALFUN_DICT.keys():
                _real_dealfun = self._CMD_DEALFUN_DICT[cmd]
        else:
            # 不区分大小写
            if cmd.lower() in self._CMD_DEALFUN_DICT.keys():
                _real_dealfun = self._CMD_DEALFUN_DICT[cmd.lower()]

        # 执行函数
        if _real_dealfun is not None:
            return _real_dealfun(message=message, cmd=cmd, cmd_para=cmd_para, **kwargs)
        else:
            return _("'$1' is not support command!", (cmd, ))

    #############################
    # 指令处理函数
    #############################
    def _help_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        帮助命令处理函数

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        # 命令相关参数
        _CMD_HELP_INFO = self._console_global_para['CMD_HELP_INFO']
        _lang = self._console_global_para['language']

        _cmd_list = PromptPlus.get_cmd_para_list(cmd_para)
        if len(_cmd_list) == 0:
            # 没有传入任何参数，生成命令提示参数并返回
            _cmd_list_tips = ''
            for _key in _CMD_HELP_INFO.keys():
                _cmd_title = ''
                if _lang in _CMD_HELP_INFO[_key].keys():
                    _cmd_title = _CMD_HELP_INFO[_key][_lang][0]
                elif 'en' in _CMD_HELP_INFO[_key].keys():
                    _cmd_title = _CMD_HELP_INFO[_key]['en'][0]
                else:
                    _cmd_title = _CMD_HELP_INFO[_key][list(_CMD_HELP_INFO[_key].keys())[0]][0]
                _cmd_list_tips = '%s%s%s\r\n' % (
                    _cmd_list_tips,
                    StringTool.fill_fix_string(_key, 15, ' ', left=False),
                    _cmd_title
                )
            # 返回提示信息
            return _cmd_list_tips

        # 处理某个命令的具体帮助信息
        _para_value = ''
        for _item in _cmd_list:
            if _item[0] == '':
                _para_value = _item[1]
                break
            elif _item[0] == '-c' or _item[0] == '-cmd':
                _para_value = _item[1]
                break

        if _para_value in _CMD_HELP_INFO.keys():
            # 显示指定命令帮助
            _help_info = None
            if _lang in _CMD_HELP_INFO[_para_value].keys():
                _help_info = _CMD_HELP_INFO[_para_value][_lang]
            elif 'en' in _CMD_HELP_INFO[_para_value].keys():
                _help_info = _CMD_HELP_INFO[_para_value]['en']
            else:
                _help_info = _CMD_HELP_INFO[_para_value][list(
                    _CMD_HELP_INFO[_para_value].keys())[0]]
            return '\r\n'.join(_help_info)
        else:
            # 没有匹配到命令
            return _("help not support '$1' command\r\n", (_para_value, ))

    def _syscmd_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        执行操作系统命令处理函数

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        _shell_encoding = self._console_global_para['shell_encoding']
        _cmd_list = PromptPlus.get_cmd_para_list(cmd_para)
        _is_start_cmd = False
        _is_ni = False
        _is_bg = False
        _is_ns = False
        _real_cmd = ''

        for _item in _cmd_list:
            if not _is_start_cmd:
                # 还要判断是否参数
                if _item[0] in ['-ni', '-bg', '-ns']:
                    if _item[0] == '-ni':
                        _is_ni = True
                    elif _item[0] == '-bg':
                        _is_bg = True
                    else:
                        _is_ns = True

                    # 判断是否开始了命令
                    if _item[1] != '':
                        _real_cmd = _item[1]
                        _is_start_cmd = True

                    # 处理下一个对象
                    continue
                else:
                    # 开始是正式的命令
                    _is_start_cmd = True

            # 开始命令部分的处理
            _real_cmd += (' %s%s%s' % (
                _item[0],
                (' ' if _item[0] != '' and _item[0][0: 1] == '-' and _item[1] != '' else ''),
                _item[1]
            ))

        # 去掉前面的空格
        _real_cmd = _real_cmd.lstrip()

        # 执行进程
        yield '%s: %s\r\n\r\n' % (_('Execute system command'), _real_cmd)
        _sp = subprocess.Popen(
            _real_cmd, close_fds=_is_bg,
            stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            shell=True
        )

        # 循环等待执行完成
        _exit_code = None
        try:
            while True:
                try:
                    # 打印内容
                    if not _is_ns:
                        yield _sp.stdout.readline().decode(_shell_encoding)
                    _exit_code = _sp.poll()
                    if _exit_code is not None:
                        # 结束，打印异常日志
                        if not _is_ns:
                            yield _sp.stdout.read().decode(_shell_encoding)
                            if _exit_code != 0:
                                yield _sp.stderr.read().decode(_shell_encoding)
                        break
                    # 释放一下CPU
                    time.sleep(0.01)
                except KeyboardInterrupt:
                    if _is_ni:
                        # 不允许取消
                        yield '\r\n' + _("ommand Executing, can't exit execute job!")
                    else:
                        raise KeyboardInterrupt
        except KeyboardInterrupt:
            # 遇到 Ctrl + C 退出
            if not _is_bg:
                # 结束任务
                _sp.terminate()
                _exit_code = _sp.poll()
                yield '\r\n' + _("Cancle Command execute, exit job!")
            else:
                yield '\r\n' + _("Command execute in bg, exit monitor!")

        # 最后返回
        if _exit_code is not None:
            yield '\r\n%s : %d' % (_("Command done, exit code"), _exit_code)

        yield '\r\n'

    def _pwd_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        显示当前工作目录命令

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        return _('Current work path is: $1', self._console_global_para['work_path'])

    def _cd_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        切换当前工作目录命令

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        _word_path = cmd_para.strip()
        os.chdir(_word_path)  # 改变工作目录，如果没有异常则认为成功
        self._console_global_para['work_path'] = os.getcwd()
        return _('Current work path is: $1', self._console_global_para['work_path'])

    def _setlanguage_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        修改显示语言

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        _lang = cmd_para.strip()
        if _lang == '':
            return _('language must be not null')
        self._console_global_para['language'] = cmd_para.strip()
        _i18n_obj = get_global_i18n()
        if _i18n_obj is not None:
            _i18n_obj.lang = self._console_global_para['language']
        # 修改配置文件中的默认语言
        _config_xml = SimpleXml(
            self._console_global_para['config_file'],
            encoding=self._console_global_para['config_encoding']
        )
        _config_xml.set_value(
            '/console/language',
            self._console_global_para['language']
        )
        _config_xml.save()
        return _("Current language set to '$1'", self._console_global_para['language'])


if __name__ == '__main__':
    # 当程序自己独立运行时执行的操作
    # 打印版本信息
    print(('模块名：%s  -  %s\n'
           '作者：%s\n'
           '发布日期：%s\n'
           '版本：%s' % (__MOUDLE__, __DESCRIPT__, __AUTHOR__, __PUBLISH__, __VERSION__)))



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2019 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import sys
import os
import copy
import subprocess
import time
from HiveNetLib.base_tools.string_tool import StringTool
from HiveNetLib.base_tools.file_tool import FileTool
from HiveNetLib.base_tools.run_tool import RunTool
from HiveNetLib.prompt_plus import PromptPlus
from HiveNetLib.base_tools.import_tool import ImportTool
from HiveNetLib.simple_i18n import _, SimpleI18N, set_global_i18n
# 根据当前文件路径将包路径纳入，在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))

"""
控制台服务框架模块
@module server
@file server.py
"""

__MOUDLE__ = 'server'  # 模块名
__DESCRIPT__ = u'控制台服务框架模块'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2019.12.3'  # 发布日期


class ConsoleServer(object):
    """
    控制台服务框架
    """
    #############################
    # 内部变量
    #############################
    _import_object_dict = None  # 登记已经导入的对象字典
    _CMD_HELP_INFO = None
    _CMD_PARA = None

    #############################
    # 构造函数
    #############################

    def __init__(self, config_dict):
        """
        初始化构造函数

        @param {dict} server_config_dict - 服务器的初始化参数字典（console节点）
        """
        # 初始化参数
        self._config_dict = copy.deepcopy(config_dict)

        # 将部分公共参数放到全局变量
        self._console_global_para = RunTool.get_global_var('CONSOLE_GLOBAL_PARA')
        self._console_global_para['name'] = self._config_dict['name']
        self._console_global_para['language'] = self._config_dict['language']
        self._console_global_para['shell_encoding'] = self._config_dict['shell_encoding']  # 控制台编码

        # i18n多语言加载
        _trans_file_path = None
        if self._config_dict['i18n'] == '':
            _trans_file_path = os.path.join(
                self._console_global_para['execute_file_path'], 'i18n/'
            )
        else:
            _trans_file_path = self._config_dict['i18n']
        _i18n_obj = SimpleI18N(
            lang=self._config_dict['language'],
            trans_file_path=_trans_file_path,
            trans_file_prefix='message',
            auto_loads=True
        )
        set_global_i18n(_i18n_obj)

        # 装载默认执行函数
        _temp_dict = self._config_dict['default_dealfun']
        self._default_cmd_dealfun = self._import_and_init_class(
            _temp_dict['module_name'],
            _temp_dict['class_name'],
            _temp_dict['extend_path'],
            _temp_dict['init_para']
        ).cmd_dealfun

        _temp_dict = self._config_dict['on_abort']
        self._on_abort = self._import_and_init_class(
            _temp_dict['module_name'],
            _temp_dict['class_name'],
            _temp_dict['extend_path'],
            _temp_dict['init_para']
        ).cmd_dealfun

        _temp_dict = self._config_dict['on_exit']
        self._on_exit = self._import_and_init_class(
            _temp_dict['module_name'],
            _temp_dict['class_name'],
            _temp_dict['extend_path'],
            _temp_dict['init_para']
        ).cmd_dealfun

        # 遍历参数装载
        self._init_cmd_paras()

        # 加入到CONSOLE_GLOBAL_PARA参数中
        self._console_global_para['CMD_PARA'] = self._CMD_PARA
        self._console_global_para['CMD_HELP_INFO'] = self._CMD_HELP_INFO

        # 控制台启动时的提示语言
        self._CONSOLE_TIPS = StringTool.json_to_object(self._config_dict['start_tips'])

        # 初始化命令行工具对象
        self._prompt = PromptPlus(
            message=self._config_dict['message'],
            default='',  # 默认输入值
            cmd_para=self._CMD_PARA,  # 命令定义参数
            default_dealfun=self._default_cmd_dealfun,  # 默认处理函数
            on_abort=self._on_abort,  # Ctrl + C 取消本次输入执行函数
            on_exit=self._on_exit  # Ctrl + D 关闭命令行执行函数
        )

    #############################
    # 公共函数
    #############################
    def start_console(self):
        """
        启动控制台服务

        @returns {CResult} - 启动结果，result.code：'00000'-成功，'21401'-服务不属于停止状态，不能启动，其他-异常
        """
        _lang = self._console_global_para['language']
        _tips = ''
        # 如果找不到对应的语言，优先找英语，如果再找不到就找第一个
        if _lang in self._CONSOLE_TIPS.keys():
            _tips = self._CONSOLE_TIPS[_lang]
        elif 'en' in self._CONSOLE_TIPS.keys():
            _tips = self._CONSOLE_TIPS['en']
        else:
            _tips = self._CONSOLE_TIPS[self._CONSOLE_TIPS.keys()[0]]

        self._prompt.start_prompt_service(
            tips='\r\n'.join(_tips).replace('{{VERSION}}', self._config_dict['version']),
            is_async=False,
            is_print_async_execute_info=True
        )

    #############################
    # 内部函数
    #############################
    def _import_and_init_class(self, module_name, class_name, extend_path, init_para):
        """
        装载并初始化对象返回（如果对象已存在则直接返回）

        @param {string} module_name - 模块名
        @param {string} class_name - 处理类名
        @param {string} extend_path - 模块所在搜索路径
        @param {string} init_para - 初始化的json字符串

        @return {object} - 初始化后的模块对象

        @throws {ImportError} - 初始化失败返回该异常
        """
        # 检查对象是否已存在
        _key = '%s.%s' % (module_name, class_name)
        if self._import_object_dict is None:
            self._import_object_dict = dict()
        if _key in self._import_object_dict.keys():
            # 已存在，直接返回即可
            return self._import_object_dict[_key]

        # 装载模块
        _class = None
        if ImportTool.check_moudle_imported(module_name):
            # 模块已存在
            _class = ImportTool.get_member_from_moudle(
                ImportTool.get_imported_moudle(module_name),
                class_name
            )
        else:
            # 动态装载模块
            _class = ImportTool.get_member_from_moudle(
                ImportTool.import_module(
                    module_name,
                    extend_path=extend_path
                ),
                class_name
            )

        if _class is None:
            raise ImportError(_("config file error: can't import module: $1!", (_key, )))

        # 初始化对象并返回
        _init_para = dict()
        if init_para != '':
            _init_para = StringTool.json_to_object(init_para)
        self._import_object_dict[_key] = _class(**_init_para)
        return self._import_object_dict[_key]

    def _init_cmd_paras(self):
        """
        初始化控制台参数
        实现self._CMD_HELP_INFO和self._CMD_PARA字典的装载
        """
        if self._CMD_HELP_INFO is None:
            self._CMD_HELP_INFO = dict()
        else:
            self._CMD_HELP_INFO.clear()

        if self._CMD_PARA is None:
            self._CMD_PARA = dict()
        else:
            self._CMD_PARA.clear()

        # 遍历cmd_list进行装载命令参数
        for _item in self._config_dict['cmd_list']:
            try:
                # 帮助信息
                self._CMD_HELP_INFO[_item['command']] = StringTool.json_to_object(_item['help'])
            except Exception as e:
                print('config file cmd [%s] help json string error!' % _item['command'])
                raise e

            try:
                # 命令信息
                self._CMD_PARA[_item['command']] = StringTool.json_to_object(_item['cmd_para'])
            except Exception as e:
                print('config file cmd [%s] cmd_para json string error!' % _item['command'])
                raise e

            # 处理函数
            self._CMD_PARA[_item['command']]['deal_fun'] = self._import_and_init_class(
                _item['module_name'],
                _item['class_name'],
                _item['extend_path'],
                _item['init_para']
            ).cmd_dealfun


if __name__ == '__main__':
    # 当程序自己独立运行时执行的操作
    # 打印版本信息
    print(('模块名：%s  -  %s\n'
           '作者：%s\n'
           '发布日期：%s\n'
           '版本：%s' % (__MOUDLE__, __DESCRIPT__, __AUTHOR__, __PUBLISH__, __VERSION__)))



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
#
#  Copyright 2019 黎慧剑
#
#  This Source Code Form is subject to the terms of the Mozilla Public
#  License, v. 2.0. If a copy of the MPL was not distributed with this
#  file, You can obtain one at http://mozilla.org/MPL/2.0/.


"""The setup.py file for Python HiveNetLib."""

from setuptools import setup, find_packages


LONG_DESCRIPTION = """
mediawikiTool is a command line tool for mediawiki file format covert and so on
""".strip()

SHORT_DESCRIPTION = """
A command line tool for mediawiki file format covert and so on.""".strip()

DEPENDENCIES = [
    'HiveNetLib>=0.7.2',
]

# DEPENDENCIES = []

TEST_DEPENDENCIES = []

VERSION = '0.5.1'
URL = 'https://github.com/snakeclub/mediawikiTool'

setup(
    # pypi中的名称，pip或者easy_install安装时使用的名称
    name="mediawikiTool",
    version=VERSION,
    author="黎慧剑",
    author_email="snakeclub@163.com",
    maintainer='黎慧剑',
    maintainer_email='snakeclub@163.com',
    description=SHORT_DESCRIPTION,
    long_description=LONG_DESCRIPTION,
    license="Mozilla Public License 2.0",
    keywords="MediaWiki Tool",
    url=URL,
    platforms=["all"],
    # 需要打包的目录列表, 可以指定路径packages=['path1', 'path2', ...]
    packages=find_packages(),
    install_requires=DEPENDENCIES,
    tests_require=TEST_DEPENDENCIES,
    package_data={'': ['*.json', '*.xml']},  # 这里将打包所有的json文件
    classifiers=[
        'Operating System :: OS Independent',
        'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
        'Programming Language :: Python :: 3.6',
        'Topic :: Software Development :: Libraries'
    ],
    entry_points={'console_scripts': [
        "wikitool = mediawikiTool.console:main"
    ]},
    # 此项需要，否则卸载时报windows error
    zip_safe=False
)



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2019 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
mediawikiTool Console (控制台)
@module console
@file console.py
"""

import sys
import os
from HiveNetLib.base_tools.run_tool import RunTool
from HiveNetLib.base_tools.string_tool import StringTool
from HiveNetLib.base_tools.file_tool import FileTool
from HiveNetLib.simple_xml import SimpleXml
# 根据当前文件路径将包路径纳入，在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(os.path.dirname(__file__), os.path.pardir)))
from mediawikiTool.lib.server import ConsoleServer


__MOUDLE__ = 'console'  # 模块名
__DESCRIPT__ = u'mediawikiTool Console (控制台)'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2019.12.3'  # 发布日期


def main(**kwargs):
    # 获取命令行参数，需要外部传入config、encoding参数
    # 例如 console.py config=/conf/config.xml encoding=utf-8
    CONSOLE_GLOBAL_PARA = {
        # 'execute_file_path': FileTool.get_exefile_path(),  # 执行文件所在目录，应指当前文件
        'execute_file_path': os.path.realpath(FileTool.get_file_path(__file__)),
        'work_path': os.getcwd(),  # 工作目录，可以通过cd命令切换，通过pwd命令查看工作目录路径
    }
    RunTool.set_global_var('CONSOLE_GLOBAL_PARA', CONSOLE_GLOBAL_PARA)
    _cmd_opts = RunTool.get_kv_opts()
    _config = (
        os.path.join(CONSOLE_GLOBAL_PARA['execute_file_path'], 'conf/config.xml')
    ) if 'config' not in _cmd_opts.keys() else _cmd_opts['config']
    _encoding = 'utf-8' if 'encoding' not in _cmd_opts.keys() else _cmd_opts['encoding']
    CONSOLE_GLOBAL_PARA['config_encoding'] = _encoding
    CONSOLE_GLOBAL_PARA['config_file'] = _config

    # 获取配置文件信息
    _config_xml = SimpleXml(os.path.realpath(_config), encoding=_encoding)
    _config_dict = _config_xml.to_dict()

    # 启动控制台服务
    _server = ConsoleServer(_config_dict['console'])
    _server.start_console()


if __name__ == '__main__':
    main()



#!/usr/bin/env python3
# -*- coding: UTF-8 -*-
# Copyright 2019 黎慧剑
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

"""
mediawiki工具命令模块
@module mediawiki_cmd
@file mediawiki_cmd.py
"""

import os
import sys
import re
import shutil
import requests
import traceback
import platform
import subprocess
import time
try:
    import chardet
except:
    pass
from HiveNetLib.base_tools.run_tool import RunTool
from HiveNetLib.prompt_plus import PromptPlus
from HiveNetLib.base_tools.file_tool import FileTool
from HiveNetLib.base_tools.string_tool import StringTool
from HiveNetLib.simple_i18n import _
# 根据当前文件路径将包路径纳入，在非安装的情况下可以引用到
sys.path.append(os.path.abspath(os.path.join(
    os.path.dirname(__file__), os.path.pardir, os.path.pardir)))
from mediawikiTool.lib.base_cmd import CmdBaseFW


__MOUDLE__ = 'mediawiki_cmd'  # 模块名
__DESCRIPT__ = u'mediawiki工具命令模块'  # 模块描述
__VERSION__ = '0.1.0'  # 版本
__AUTHOR__ = u'黎慧剑'  # 作者
__PUBLISH__ = '2019.12.09'  # 发布日期


class MediaWikiCmd(CmdBaseFW):
    """
    通用命令处理
    支持help、syscmd命令
    """
    #############################
    # 构造函数，在里面增加函数映射字典
    #############################

    def _init(self, **kwargs):
        """
        实现类需要覆盖实现的初始化函数

        @param {kwargs} - 传入初始化参数字典（config.xml的init_para字典）

        @throws {exception-type} - 如果初始化异常应抛出异常
        """
        self._CMD_DEALFUN_DICT = {
            'mdtowiki': self._mdtowiki_cmd_dealfun,
            'docxtowiki': self._docxtowiki_cmd_dealfun,
            'filestowiki': self._filestowiki_cmd_dealfun
        }
        self._console_global_para = RunTool.get_global_var('CONSOLE_GLOBAL_PARA')

    #############################
    # 实际处理函数
    #############################
    def _cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        通用处理函数，通过cmd区别调用实际的处理函数

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）
        @param {kwargs} - 传入的主进程的初始化kwargs对象

        @return {string|iter_string} - 执行命令完成后要输到屏幕的内容，可以是iter对象（yield返回）
            注：如果是on_abort和on_exit可以返回CResult对象
        """
        # 获取真实执行的函数
        _real_dealfun = None  # 真实调用的函数
        if 'ignore_case' in kwargs.keys() and kwargs['ignore_case']:
            # 区分大小写
            if cmd in self._CMD_DEALFUN_DICT.keys():
                _real_dealfun = self._CMD_DEALFUN_DICT[cmd]
        else:
            # 不区分大小写
            if cmd.lower() in self._CMD_DEALFUN_DICT.keys():
                _real_dealfun = self._CMD_DEALFUN_DICT[cmd.lower()]

        # 执行函数
        if _real_dealfun is not None:
            return _real_dealfun(message=message, cmd=cmd, cmd_para=cmd_para, **kwargs)
        else:
            return _("'$1' is not support command!", (cmd, ))

    #############################
    # 实际处理函数
    #############################
    def _mdtowiki_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        将markdown格式文件转换为mediawiki格式

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        try:
            if not self._para_dict_check(cmd=cmd, cmd_para=cmd_para):
                return ''

            # 展示处理信息
            print(
                '%s ( %s ):' % (
                    _('convert info'),
                    _('change stander pic name' if self._para_dict['stdpic']
                      else 'use source pic name')
                )
            )
            print('  %s: %s' % (_('source file'), self._para_dict['in']))
            print('  %s: %s' % (_('out path'), self._para_dict['out']))
            print('  %s: %s' % (_('wiki page title'), self._para_dict['name']))
            print('  %s: %s' % (_('copy pic path'), self._para_dict['pic_dir']))
            print('')
            print('%s  =================>' % (_('begin convert'), ))
            print('')

            # 删除原有复制图片
            self._create_pic_dir()

            # 预处理markdown文件
            print('\r\n%s %s: ' % (_('begin'), _('copy pic file')))
            _md_text = FileTool.get_file_text(self._para_dict['in'], encoding=None)
            _temp_text = re.sub(
                r'!\[.*?\]\(.*?\)|\<img .*? /\>',
                self._deal_md_pic,
                _md_text
            )
            print('%s %s' % (_('copy pic file'), _('done')))
            with open(
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '_temp.md'),
                "w", encoding='utf-8'
            ) as f:
                f.write(_temp_text)
            print('\r\n%s %s' % (_('pre convert to temp file'), _('done')))

            # 调用Pandoc进行转换处理
            print('%s:' % (_('use Pandoc convert'), ))
            if platform.system() == 'Windows':
                # Win平台要先执行chcp 65001命令
                _sys_cmd = 'chcp 65001'
                self._console_global_para['shell_encoding'] = 'utf-8'
                print('%s: %s' % (_('execute'), _sys_cmd))
                if(self._exe_syscmd(_sys_cmd, shell_encoding='utf-8') != 0):
                    return ''

            # 执行转换命令
            _sys_cmd = 'pandoc %s -f markdown -t mediawiki -s -o %s' % (
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '_temp.md'),
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '.txt')
            )
            print('%s: %s' % (_('execute'), _sys_cmd))
            if(self._exe_syscmd(_sys_cmd, shell_encoding='utf-8') != 0):
                return ''

            # 删除临时文件
            FileTool.remove_file(os.path.join(
                self._para_dict['out'], self._para_dict['name'] + '_temp.md'))

            print('\r\n=================>  %s %s' % (_('convert'), _('done')))
        except Exception as e:
            _prin_str = '%s (%s):\r\n%s' % (
                _('execution exception'), str(e), traceback.format_exc()
            )
            print(_prin_str)

        # 结束
        return ''

    def _docxtowiki_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        将docx格式文件转换为mediawiki格式

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        try:
            if not self._para_dict_check(cmd=cmd, cmd_para=cmd_para):
                return ''

            self._para_dict['stdpic'] = True

            # 展示处理信息
            print(
                '%s:' % (
                    _('convert info')
                )
            )
            print('  %s: %s' % (_('source file'), self._para_dict['in']))
            print('  %s: %s' % (_('out path'), self._para_dict['out']))
            print('  %s: %s' % (_('wiki page title'), self._para_dict['name']))
            print('  %s: %s' % (_('copy pic path'), self._para_dict['pic_dir']))
            print('')
            print('%s  =================>' % (_('begin convert'), ))
            print('')

            # 删除原有复制图片
            self._create_pic_dir()

            # 先将docx转换为md
            print('%s: ' % (_('change docx file to markdown'), ))
            if platform.system() == 'Windows':
                # Win平台要先执行chcp 65001命令
                _sys_cmd = 'chcp 65001'
                self._console_global_para['shell_encoding'] = 'utf-8'
                print('%s: %s' % (_('execute'), _sys_cmd))
                if(self._exe_syscmd(_sys_cmd, shell_encoding='utf-8') != 0):
                    return ''

            _sys_cmd = 'pandoc %s -f docx -t markdown -s -o %s --extract-media=%s' % (
                self._para_dict['in'],
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '_temp.md'),
                self._para_dict['pic_dir']
            )
            print('%s: %s' % (_('execute'), _sys_cmd))
            if(self._exe_syscmd(_sys_cmd, shell_encoding='utf-8') != 0):
                return ''

            # 预处理markdown文件
            print('\r\n%s %s: ' % (_('begin'), _('begin copy pic file')))
            _md_text = FileTool.get_file_text(os.path.join(
                self._para_dict['out'], self._para_dict['name'] + '_temp.md'), encoding=None)
            _temp_text = re.sub(
                r'(!\[.*?\]\(.*?\)|\<img .*? /\>)(\{[width=|height=][\s\S]*?\})*',
                self._deal_md_pic,
                _md_text
            )

            print('%s %s' % (_('copy pic file'), _('done')))
            with open(
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '_temp.md'),
                "w", encoding='utf-8'
            ) as f:
                f.write(_temp_text)
            print('\r\n%s %s' % (_('pre convert to temp file'), _('done')))

            # 调用Pandoc进行转换处理
            print('\r\n%s:' % (_('use Pandoc convert'), ))

            # 执行转换命令
            _sys_cmd = 'pandoc %s -f markdown -t mediawiki -s -o %s' % (
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '_temp.md'),
                os.path.join(self._para_dict['out'], self._para_dict['name'] + '.txt')
            )
            print('%s: %s' % (_('execute'), _sys_cmd))
            if(self._exe_syscmd(_sys_cmd, shell_encoding='utf-8') != 0):
                return ''

            # 删除临时文件
            FileTool.remove_file(os.path.join(
                self._para_dict['out'], self._para_dict['name'] + '_temp.md'))
            FileTool.remove_dir(os.path.join(self._para_dict['pic_dir'], 'media'))

            print('\r\n=================>  %s %s' % (_('convert'), _('done')))
        except Exception as e:
            _prin_str = '%s (%s):\r\n%s' % (
                _('execution exception'), str(e), traceback.format_exc()
            )
            print(_prin_str)

        # 结束
        return ''

    def _filestowiki_cmd_dealfun(self, message='', cmd='', cmd_para='', **kwargs):
        """
        将指定目录下的文件批量转换为mediawiki格式

        @param {string} message='' - prompt提示信息
        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 执行命令完成后要输到屏幕的内容
        """
        try:
            # 获取参数及处理参数
            _cmd_list = PromptPlus.get_cmd_para_list(cmd_para)
            _in = ''
            _out = ''
            _stdpic = False
            for _item in _cmd_list:
                if '-in' == _item[0]:
                    _in = _item[1].strip("'")
                elif '-out' == _item[0]:
                    _out = _item[1].strip("'")
                elif '-stdpic' == _item[0]:
                    _stdpic = True

            if _in == '':
                _in = self._console_global_para['work_path']
            else:
                if not os.path.exists(_in) or not os.path.isdir(_in):
                    print(_("Path '$1' not exists, please check [-in] para!", _in))
                    return ''
                _in = os.path.realpath(_in)
            if _out == '':
                _out = self._console_global_para['work_path']
            else:
                if not os.path.exists(_out):
                    # 创建对应目录
                    FileTool.create_dir(_out)
                _out = os.path.realpath(_out)

            # 遍历文件并进行处理
            print(_("begin convert files in $1", _in) + ' =======================>')
            print('')
            _file_list = FileTool.get_filelist(path=_in, is_fullname=True)
            for _file in _file_list:
                _ext = FileTool.get_file_ext(_file)
                if _ext == 'md':
                    print('%s: %s' % (_('convert'), _file))
                    self._mdtowiki_cmd_dealfun(
                        message=message, cmd='mdtowiki',
                        cmd_para="-in '%s' -out '%s'%s" % (
                            _file, _out, ' -stdpic' if _stdpic else ''
                        ),
                        **kwargs
                    )
                elif _ext == 'docx':
                    print('%s: %s' % (_('convert'), _file))
                    self._docxtowiki_cmd_dealfun(
                        message=message, cmd='docxtowiki',
                        cmd_para="-in '%s' -out '%s'" % (
                            _file, _out
                        ),
                        **kwargs
                    )
                else:
                    print('%s: %s' % (_('not support file format'), _file))

            # 处理完成
            print('\r\n=======================>  %s %s' % (_('convert files'), _('done')))
        except Exception as e:
            _prin_str = '%s (%s):\r\n%s' % (
                _('execution exception'), str(e), traceback.format_exc()
            )
            print(_prin_str)

        # 结束
        return ''

    #############################
    # 内部函数
    #############################
    def _para_dict_check(self, cmd='', cmd_para=''):
        """
        检查并生成

        @param {string} cmd - 执行的命令key值
        @param {string} cmd_para - 传入的命令参数（命令后的字符串，去掉第一个空格）

        @return {string} - 如果返回空字符串代表成功，有值的字符串代表失败
        """
        # 获取命令执行参数
        _cmd_list = PromptPlus.get_cmd_para_list(cmd_para)
        self._para_dict = {
            'in': '',
            'out': '',
            'name': '',
            'stdpic': False,
            'pic_dir': '',
            'pic_list': {},
            'pic_num': 0
        }
        for _item in _cmd_list:
            if '-in' == _item[0]:
                self._para_dict['in'] = _item[1].strip("'")
            elif '-out' == _item[0]:
                self._para_dict['out'] = _item[1].strip("'")
            elif '-name' == _item[0]:
                self._para_dict['name'] = _item[1].strip("'")
            elif '-stdpic' == _item[0]:
                self._para_dict['stdpic'] = True

        # 参数检查及初始化
        if self._para_dict['in'] == '' or not os.path.exists(self._para_dict['in']) or not os.path.isfile(self._para_dict['in']):
            # 输入文件不存在
            print(_('File \'$1\' not exists, please check [-in] para!', self._para_dict['in']))
            return False

        self._para_dict['in'] = os.path.realpath(self._para_dict['in'])  # 获取全路径
        self._para_dict['in_dir'] = FileTool.get_file_path(self._para_dict['in'])

        if self._para_dict['out'] == '':
            self._para_dict['out'] = self._console_global_para['work_path']  # 使用工作路径
        else:
            if not os.path.exists(self._para_dict['out']):
                # 创建对应目录
                FileTool.create_dir(self._para_dict['out'])
            self._para_dict['out'] = os.path.realpath(self._para_dict['out'])

        if self._para_dict['name'] == '':
            self._para_dict['name'] = FileTool.get_file_name_no_ext(self._para_dict['in'])

        self._para_dict['pic_dir'] = os.path.join(
            self._para_dict['out'], self._para_dict['name'] + '_copy_pic')

        return True

    def _deal_md_pic(self, match_str):
        """
        针对获取到的md图片路径字符串进行文件处理和替换

        @param {re.Match} match_str - 匹配到的对象

        @return {string} - 替换后的图片字符串
        """
        _str = match_str.group()
        # 提取图片路径信息
        _alt = ''
        _src = ''
        _name = ''
        if _str.startswith('!['):
            # ![img004](mdtowiki_pic/img004.png)的格式
            _p_alt = re.compile(r'!\[(.*?)\]')
            _p_src = re.compile(r'\((.*?)\)')
            _alt = re.findall(_p_alt, _str)[0]
            _src = re.findall(_p_src, _str)[0]
        else:
            # <img src="https://yt-adp.ws.126.net/channel4/1200125_pads_20190404.jpg" alt="163" style="zoom:33%;" />的格式
            _p_alt = re.compile(r'\salt=(.*?)\s')
            _p_src = re.compile(r'\ssrc=(.*?)\s')
            _temp = re.findall(_p_alt, _str)
            if len(_temp) > 0:
                _alt = _temp[0].strip('\'"')
            _temp = re.findall(_p_src, _str)
            if len(_temp) > 0:
                _src = _temp[0].strip('\'"')

        # 检查文件是否已经处理过
        if _src in self._para_dict['pic_list']:
            _name = self._para_dict['pic_list'][_src]
            print('%s: %s -> %s %s' % (_('copy pic file'), _src, _name, _('done')))
        else:
            # 未处理过
            if self._para_dict['stdpic']:
                # 按顺序命名
                self._para_dict['pic_num'] = self._para_dict['pic_num'] + 1
                _name = '%s_%s_%s.%s' % (
                    self._para_dict['name'], _('embed'),
                    StringTool.fill_fix_string(str(self._para_dict['pic_num']), 5, '0'),
                    FileTool.get_file_ext(_src)
                )
            else:
                # 使用原名称
                _temp_str = _src.replace('\\', '/').replace(' ', '_')
                _index = _temp_str.rfind("/")
                if _index == -1:
                    _name = _temp_str
                else:
                    _name = _temp_str[_index + 1:]

                _name = '%s_%s_%s' % (
                    self._para_dict['name'], _('embed'), _name
                )

            # 加入到清单中
            self._para_dict['pic_list'][_src] = _name

            # 复制或下载文件
            try:
                self._down_md_pic(_src, os.path.join(self._para_dict['pic_dir'], _name))
                print('%s: %s -> %s %s' % (_('copy pic file'), _src, _name, _('done')))
            except Exception as e:
                # 提示
                print(
                    '%s: %s -> %s %s ( %s ):\r\n %s' % (
                        _('copy pic file'), _src, _name, _('execution exception'), str(e),
                        traceback.format_exc()
                    )
                )

        # 改写文件
        if _alt == '':
            return '[[Image:%s]]' % (_name, )
        else:
            return '[[Image:%s|%s]]' % (_name, _alt)

    def _down_md_pic(self, src, dest):
        """
        将图片文件下载到指定路径

        @param {string} src - 要下载的文件路径
        @param {string} dest - 目标文件
        """
        if src.startswith('http://') or src.startswith('https://'):
            # 网络图片，需要下载
            _resp = requests.get(src)
            with open(dest, "wb") as f:
                f.write(_resp.content)
        else:
            # 本地文件，进行复制
            _src_file = os.path.join(self._para_dict['in_dir'], src)
            shutil.copyfile(_src_file, dest)

    def _exe_syscmd(self, cmd, shell_encoding='utf-8'):
        """
        执行系统命令

        @param {string} cmd - 要执行的命令
        @param {string} shell_encoding='utf-8' - 界面编码

        @return {int} - 返回执行结果
        """
        _sp = subprocess.Popen(
            cmd, close_fds=True,
            stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE,
            shell=True
        )
        # 循环等待执行完成
        _exit_code = None
        try:
            while True:
                try:
                    # 打印内容
                    _show_str = _sp.stdout.readline().decode(shell_encoding).strip()
                    if _show_str != '':
                        print(_show_str)

                    _exit_code = _sp.poll()
                    if _exit_code is not None:
                        # 结束，打印异常日志
                        _show_str = _sp.stdout.read().decode(shell_encoding).strip()
                        if _show_str != '':
                            print(_show_str)
                        if _exit_code != 0:
                            _show_str = _sp.stderr.read().decode(shell_encoding).strip()
                            if _show_str != '':
                                print(_show_str)
                        break
                    # 释放一下CPU
                    time.sleep(0.01)
                except KeyboardInterrupt:
                    # 不允许取消
                    print(_("Command Executing, can't exit execute job!"))
        except KeyboardInterrupt:
            # 遇到 Ctrl + C 退出
            pass

        # 最后返回
        if _exit_code is not None:
            if _exit_code != 0:
                # 执行错误，显示异常
                print('%s : %d' % (_("Command done, exit code"), _exit_code))
            else:
                print('%s' % (_("Command execute done"), ))

        return _exit_code

    def _create_pic_dir(self):
        """
        新建或删除图片复制目录
        """
        if os.path.exists(self._para_dict['pic_dir']):
            FileTool.remove_files(path=os.path.join(
                self._para_dict['out'], self._para_dict['name'] + '_copy_pic'))
        else:
            # 创建目录
            FileTool.create_dir(self._para_dict['pic_dir'])

        print('%s %s' % (_('make pic path'), _('done')))


if __name__ == '__main__':
    # 当程序自己独立运行时执行的操作
    # 打印版本信息
    print(('模块名：%s  -  %s\n'
           '作者：%s\n'
           '发布日期：%s\n'
           '版本：%s' % (__MOUDLE__, __DESCRIPT__, __AUTHOR__, __PUBLISH__, __VERSION__)))
